﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  [Item("SlidingWindowBestSolutionsCollection", "An object holding a collection of the best sliding window solutions.")]
  public abstract class SlidingWindowBestSolutionsCollection : Item {

    private List<ISymbolicExpressionTree> bestSolutions;
    [Storable]
    public IEnumerable<ISymbolicExpressionTree> BestSolutions {
      get { return bestSolutions; }
      set { bestSolutions = new List<ISymbolicExpressionTree>(value); }
    }

    private List<IntRange> slidingWindowPositions;
    [Storable]
    public IEnumerable<IntRange> SlidingWindowPositions {
      get { return slidingWindowPositions; }
      set { slidingWindowPositions = new List<IntRange>(value); }
    }

    [Storable]
    private IDataAnalysisProblemData problemData;
    public IDataAnalysisProblemData ProblemData {
      get { return problemData; }
      set { problemData = value; }
    }

    [Storable]
    private ISymbolicDataAnalysisExpressionTreeInterpreter interpreter;
    public ISymbolicDataAnalysisExpressionTreeInterpreter Interpreter {
      get { return interpreter; }
      set { interpreter = value; }
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }

    [StorableConstructor]
    protected SlidingWindowBestSolutionsCollection(bool deserializing) : base(deserializing) { }
    protected SlidingWindowBestSolutionsCollection(SlidingWindowBestSolutionsCollection original, Cloner cloner)
      : base(original, cloner) {
      this.bestSolutions = original.bestSolutions;
      this.slidingWindowPositions = original.slidingWindowPositions;
      this.problemData = original.problemData;
      this.interpreter = original.interpreter;
    }
    protected SlidingWindowBestSolutionsCollection() {
      bestSolutions = new List<ISymbolicExpressionTree>();
      slidingWindowPositions = new List<IntRange>();
    }

    public void Add(ISymbolicExpressionTree solution, IntRange range) {
      if (bestSolutions == null) bestSolutions = new List<ISymbolicExpressionTree>();
      bestSolutions.Add(solution);
      if (slidingWindowPositions == null) slidingWindowPositions = new List<IntRange>();
      slidingWindowPositions.Add(range);
    }

    public void Clear() {
      if (bestSolutions != null) bestSolutions.Clear();
      if (slidingWindowPositions != null) slidingWindowPositions.Clear();
    }

    public abstract ISymbolicDataAnalysisModel CreateModel(ISymbolicExpressionTree tree, ISymbolicDataAnalysisExpressionTreeInterpreter interpreter,
     double lowerEstimationLimit = double.MinValue, double upperEstimationLimit = double.MaxValue);

    public abstract ISymbolicDataAnalysisSolution CreateSolution(ISymbolicDataAnalysisModel model,
      IDataAnalysisProblemData problemData);
  }
}
